''      By Shawn A.
''
'' This is a basic Complimentary Filter for the X and Y axis. I am using the 6DOF board which has an ADXL345 Accelerometer
'' and a ITG3200 Gyro.  Other sensors could be used but the program would have to be modified.  I suggest before using this code
'' figure out the X and Y orientations of your ACC and Gyro. On my ACC the X axis is my Roll but on my GYRO the Y axis is roll, so be carefull when combining
'' the two in the complimentary filter.
'' Also keep track of the signs. I had to multiply the GYRO Y axis by -1 to make sure the sign matched the ACC's sign.
'' At the end, the program converts the Floating Point values to integers to display in PST.
'' 
'' I have not tested it, but I suspect there will be a problem if the board is rotated more than 180 degrees because the gyro will
'' just keep adding to its angle.
'' Also there needs to be a routine to zero the ACC, I wrote one but I did not include it in this demo.  The ITG 3200 Sport pasm has a routine for zeroing the Gyro,
'' a routine for zeroing the ACC could be modeled off of that.

CON
        _clkmode = xtal1 + pll16x
        _xinfreq = 5_000_000

NegOne  = -1.0
ACCPCT  = 0.02    'ACCPCT can be changed and so can GYROPCT but there sum must equal 1.
GYROPCT = 0.98
MHZ     = 80
OHund   = 100.0
GyroLSB = 14.375
GyroDT  = 0.004   'Is the duration of the repeat loop in the main program below.  This is 250Hz.  If this is changed the waitcnt command needs
                  'changed at the bottom of the repeat loop.
OBJ
FP32F    : "Float32Full"               'By Cam Thompson
AccGyro  : "ITG 3200 Sport pasm"       'By Jason Dorie                      
PST      : "Parallax Serial Terminal"  'By Jeff Martin, Andy Lindsay, Chip Gracey

VAR
  Long Roll, Pitch              'Stores Roll and Pitch Angles as Floating Point.
  Long ARoll,APitch             'Stores ACC Pitch and Roll Calculations.
  Long GRoll,GPitch             'Stores Gyro Pitch and Roll Calculations.
  Long RollI,PitchI             'Stores Roll and Pitch Angles as Integer.
  Long Ax,Ay,Az                 'ACC readings from ADXL345
  Long Gx, Gy, Gz               'Gyro readings from ITG3200 
  Long LastTime                 'Used to keep track of 250Hz Sample Rate
  Long DisplayCounter           'Used to determine when to send data to PST
PUB Main

''''''Initializes a Couple of Things''''''
  PST.Start(115_200)        'Sets BaudRate for PST.
  PST.Clear                 'Clears PST Screen.
  FP32F.Start               'Starts Math Engine FF32.
  AccGyro.Start(6,7)        'Starts Gyro and ACC(6DOF). Sets I2C Pins and Zeros the 6DOF.
  DisplayCounter := 0
  LastTime       := cnt
''''''Since we are using a Gyro to find Angles of X and Y we need to time the repeat loop. At the end of the loop
''''''there is a waitcnt command that makes sure the loop takes 4mS, which is 250Hz.  We have to know how often we are
''''''sample the gyro for our angle calculation.  
Repeat
'
''''''Read Zeroed Values from Gyro''''''''
'Jason's code has a routine that zeros the gyro at startup, you can also get raw readings if you want.
    Gx := AccGyro.GetRX
    Gy := AccGyro.GetRY
    Gz := AccGyro.GetRZ
    
 '''''Read Raw Values from ACC''''''''
    Ax := AccGyro.GetAx
    Ay := AccGyro.GetAY
    Az := AccGyro.GetAz
   
'''''''Convert Gyro Values to Floating Point''''''''
    Gx := FP32F.FFloat(Gx)
    Gy := FP32F.FFloat(Gy)
    Gz := FP32F.FFloat(Gz)

 '''''''Convert ACC Raw Values to Floating Point''''''''
    Ax    := FP32F.FFloat(Ax)
    Ay    := FP32F.FFloat(Ay)
    Az    := FP32F.FFloat(Az)
       
 ''''''Performs Math at Specific Time to Accumulate Gyro Angle''''''
 '''''' Gyro_Y_Angle = Gyro_Y_Angle + ((Gy/GyroLSB)*dT)
    Gy     := FP32F.FMul(Gy,NegOne)                     'This reverses the sign of Gy.  
    Gy     := FP32F.FDiv(Gy,GyroLSB)
    Gy     := FP32F.FMul(Gy,GyroDT)
    GRoll  := FP32F.FAdd(Roll,Gy)
    
'''''''Performs Math at Specific Time to Accumulate Gyro Angle''''''
 '''''' Gyro_X_Angle = Gyro_X_Angle + ((Gx/GyroLSB)*dT)
    Gx     := FP32F.FDiv(Gx,GyroLSB)
    Gx     := FP32F.FMul(Gx,GyroDT)
    GPitch := FP32F.FAdd(Pitch,Gx)
 
 ''''''Performs ATAN2 Function then Converts to Degrees for X Axis''''''
 ''''''Acc_X_Angle = ATAN2(X_ AccValue,Z_AccValue)
    ARoll  := FP32F.ATan2(Ax,Az)
    ARoll  := FP32F.Degrees(ARoll)
    
 ''''''Performs ATAN2 Function then Converts to Degrees for Y Axis''''''    
 ''''''Acc_Y_Angle = ATAN2(Y_ AccValue,Z_AccValue)
    APitch := FP32F.ATan2(Ay,Az)
    APitch := FP32F.Degrees(APitch)
    
 ''''''Complimentary Filter for Roll Axis''''''
 ''''''Roll = (GRoll * GYROPCT) + (ARoll * ACCPCT)
    GRoll  := FP32F.FMul(GRoll,GYROPCT)
    ARoll  := FP32F.FMul(ARoll,ACCPCT)
    Roll   := FP32F.FAdd(GRoll,ARoll)

 ''''''Complimentary Filter for Pitch Axis''''''   
 ''''''Pitch = (GPitch * GYROPCT) + (APitch * ACCPCT)
    GPitch  := FP32F.FMul(GPitch,GYROPCT)
    APitch  := FP32F.FMul(APitch,ACCPCT)
    Pitch   := FP32F.FAdd(GPitch,APitch)

 ''''''Converts Roll and Pitch to Integers to Display in PST''''''
 'Converts Degrees into Integers, I take Roll and Pitch * 100 so that I can see 2 Decimal Places in the Result.
 'For Example 90 Degrees would Read 9000 Degrees
    RollI  := FP32F.FMul(OHund,Roll)
    PitchI := FP32F.FMul(OHund,Pitch)
'
'''''Rounds Roll and then Pitch and Converts to Integers''''''
    RollI  := FP32F.FRound(RollI)
    PitchI := FP32F.FRound(PitchI)
    
 ''''''Displays Roll and Pitch in PST Window''''''
 'If this section of code is expanded on it could throw of the 250Hz sample rate be carefull with adding to it.
    If DisplayCounter == 62     '62 is aprox. 1/4 of 250 this determines how often the data is sent to PST.Aprox. 4 times a second.
      PST.Clear
      PST.Char("X")
      PST.Char("=")
      PST.Char(" ")
      PST.Dec(RollI)
      PST.NewLine
      PST.Char("Y")
      PST.Char("=")
      PST.Char(" ")
      PST.Dec(PitchI)
      DisplayCounter := 0
    else
      DisplayCounter := DisplayCounter + 1
      
 ''''''Makes sure the repeat loop takes 4mS   
    waitcnt( constant(80_000_000 / 250) + LastTime )
    LastTime += constant(80_000_000 / 250)        